import sys
import json

try:
    import requests
except ImportError:
    sys.stderr.write("request module is needed")
    sys.exit(1)


class FacetteAPIError(Exception):
    STATUS = {
        200: """OK:
        The request was successful and the server is returning data in the
        response body.""",
        201: """Created:
        The request was successful resulting in the creation of a new item.
        The response body is empty.""",
        400: """Bad Request:
        The request or the data supplied along with the request is invalid,
        could not be processed or when mandatory fields are not met.""",
        404: """Not Found:
        The requested item or an item to inherit from could not be found.
        Could be returned by POST creation requests.""",
        405: """Method Not Allowed:
        The requested method is not supported the API resource.""",
        409: """Conflict:
        An item with the same name and type already exists.
        Could be returned by POST and PUT creation requests.""",
        415: """Unsupported Media Type:
        The request used a Content-Type not supported by the API resource.""",
        503: """Service Unavailable:
        The server is currently unavailable.
        Such response is sent when the server is being initialized or reloading
        its configuration."""
    }

    def __init__(self, status):
        self.status = status

    def __str__(self):
        return "status=%s (%s)" % (self.status,
                                   self.STATUS.get(self.status,
                                                   "Unknown error")
                                   )

    def __repr__(self):
        return "status=%s (%s)" % (self.status,
                                   self.STATUS.get(self.status,
                                                   "Unknown error")
                                   )


class FacetteBase(object):
    def __init__(self,
                 host='localhost',
                 port=80,
                 username=None,
                 password=None,
                 ssl=False,
                 verify_ssl=False,
                 timeout=None,
                 baseapi=None,
                 apiversion=1
                 ):

        self._host = host
        self._port = port
        self._username = username
        self._password = password

        self._ssl = ssl
        self._verify_ssl = verify_ssl
        self._timeout = timeout

        self._scheme = 'http'

        if ssl is True:
            self._scheme = 'https'

        if baseapi is None:
            self._baseapi = '/api/v%s/' % apiversion

        self._baseurl = "{0}://{1}:{2}{3}".format(
            self._scheme,
            self._host,
            self._port,
            self._baseapi)

        self._headers = {
            'Content-type': 'application/json',
            'Accept': 'text/plain',
            'User-Agent': 'Facette Python'}

    @staticmethod
    def _serialize(data):
        if isinstance(data, dict):
            return json.loads(data)
        else:
            return data

    def _get(self, resource, **kwargs):
        payload = kwargs
        r = requests.get(self._baseurl + resource,
                         auth=(self._username, self._password),
                         params=payload,
                         timeout=self._timeout,
                         verify=self._verify_ssl,
                         headers=self._headers)
        if not r.status_code == 200:
            raise FacetteAPIError(r.status_code)
        try:
            return {'data': self._serialize(r.text),
                    'records': r.headers['X-Total-Records']}
        except KeyError:
            return {'data': self._serialize(r.text)}

    @staticmethod
    def _get_returns(response, **kwargs):
        """
        Internal method

        :param response:
        :param kwargs:
        :return:
        """
        print_data = kwargs.get('print_data', True)
        total_record = kwargs.get('total_record', False)
        if print_data and not total_record:
            return json.loads(response['data'])
        elif not print_data and total_record:
            try:
                return json.loads(response['records'])
            except KeyError:
                pass
        elif print_data and total_record:
            return json.loads(response)

    @staticmethod
    def _parse_location(resource):
        return resource.split('/')[5]

    def _post(self, resource, data):
        payload = json.dumps(data)
        r = requests.post(self._baseurl + resource,
                          auth=(self._username, self._password),
                          timeout=self._timeout,
                          verify=self._verify_ssl,
                          headers=self._headers,
                          data=payload)
        if not r.status_code == 201:
            raise FacetteAPIError(r.status_code)
        return self._parse_location(r.headers['Location'])

    def _delete(self, resource):
        r = requests.delete(self._baseurl + resource,
                            auth=(self._username, self._password),
                            timeout=self._timeout,
                            verify=self._verify_ssl,
                            headers=self._headers)
        if not r.status_code == 200:
            raise FacetteAPIError(r.status_code)


class _GlobalWrapper(FacetteBase):
    def __init__(self, resource, **kwargs):
        FacetteBase.__init__(self, **kwargs)
        self.resource = resource

        if self.resource.split('/')[0] == 'catalog':
            self.get_all = self._get_all
            self.get_single = self._get_single
        elif self.resource.split('/')[1] == 'labels':
            self.get = self._get_units_label
        else:
            self.get_single = self._get_single
            self.get_all = self._get_all
            self.create = self._create
            self.update = self._update
            self.delete = self._delete

    def _get_units_label(self, **kwargs):
        response = self._get('%s' % self.resource)
        return self._get_returns(response, **kwargs)

    def _get_all(self, **kwargs):
        payload = dict()
        payload['filter'] = kwargs.get('filter')
        payload['limit'] = kwargs.get('limit')
        payload['offset'] = kwargs.get('offset')
        payload['templates'] = kwargs.get('templates')
        payload['collection'] = kwargs.get('collection')
        payload['exclude'] = kwargs.get('exclude')
        payload['parent'] = kwargs.get('parent')
        response = self._get(self.resource)
        return self._get_returns(response, **kwargs)

    def _get_single(self, record):
        return self._get('%s/%s' % (self.resource, record))

    def _create(self, data):
        return self._post('%s' % self.resource, data)

    def _update(self):
        pass

    def _delete(self, uuid):
        return self._delete('%s/%s' % (self.resource, uuid))
